package simFil;
/*  SimFil_Graphics.... all the Java3D, events, etc */

/*  <SimFil - simulation of cytoskeletal filament flexibility by Pairwise Agent Interaction with Rational Superposition (PAIRS)>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import infoCCD.*;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.media.j3d.*;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JSlider;
import javax.vecmath.*;
import javax.swing.event.*;

import java.io.File;
import java.net.*;
import java.text.DecimalFormat;

import com.sun.j3d.utils.universe.*;

public class SimFil_Graphics_ extends JFrame
	implements WindowListener, CheckboxMenuListener_, ActionListener, ItemListener, ChangeListener {

	//  Should the scene be compiled?
	private boolean shouldCompile = true;

	//  GUI objects for our subclasses
	protected Frame    theFrame    = null;
	protected MenuBar  theMenuBar  = null;
	protected static CapturingCanvas3D_ theCanvas = null;
	protected static JPanel topPanel = null;
	protected static JPanel bottomPanel = null;
	protected static JLabel timeLabel = null;
	protected static JLabel concLabel = null;
	protected static JLabel filCtLabel = null;
	protected static JLabel monCtLabel = null;
	protected static TransformGroup theViewTransform = null;
	protected static TransformGroup theSceneTransform = null;
	public JSlider bugTrans,bugEmit,headLightI,bugLightI;
	
	public static int frameHeadSize;		// different on different systems... size of bar at top of frame and the diff between canvas3d and JFrame height
	public static int bottomPanelHeight = 50;
	
	//  Private GUI objects and state
	public DirectionalLight headlight = null;
	public PointLight bugEnd1Light = null;
	public PointLight bugEnd2Light = null;
	private ExamineViewerBehavior_ examineBehavior = null;

	private Background background = null;
	private static Group scene = null;
	private static BranchGroup bugState = null;
	
	
	// for menus
	private static boolean headlightOn = true;
	private static boolean bugEndLightsOn = false;
	private CheckboxMenuItem paintOnMenuItem = null;
	private static CheckboxMenuItem writeFileMenuItem = null;
	private static CheckboxMenuItem logLpMenuItem = null;
	private static CheckboxMenuItem writeQKFileMenuItem = null;
	public static CheckboxMenuItem pausedMenuItem = null;
	public static CheckboxMenuItem runMenuItem = null;
	private static CheckboxMenuItem rotateMenuItem = null;
	private static CheckboxMenuItem filMotionOffMenuItem = null;
	private static CheckboxMenuItem plasmidMotionOffMenuItem = null;;
	private static CheckboxMenuItem filCoordSysOnMenuItem = null;
	private static CheckboxMenuItem relaxConstraintsMenuItem = null;
	private static CheckboxMenuItem persistenceTstMenuItem = null;
	private static CheckboxMenuItem filUnderTractionMenuItem = null;
	private static CheckboxMenuItem sphereInPlanesMenuItem = null;
	private static CheckboxMenuItem spheresInForceFieldMenuItem = null;
	private static CheckboxMenuItem spheresSqueezedMenuItem = null;
	private static CheckboxMenuItem spheresLinkedMenuItem = null;
	private static CheckboxMenuItem simpleSupportTstMenuItem = null;
	private static CheckboxMenuItem simpleSupportTstMenuItemMode2 = null;
	private static CheckboxMenuItem cantileverTstMenuItem = null;
	
	// report strings
	static String filCtString;
	static String filSegCtString;
	
	
	// control frames with organizing tab and panel
	public static JFrame controlFrame = new JFrame("Control Panel");
	public static JPanel controlPanel = new JPanel(new GridLayout(1,1));
	public static JTabbedPane controlTab = new JTabbedPane();
	public static GraphicsControl_ graphicsControl;
	public static MainControl_ mainControl;
	public static SphereControl_ sphControl;
	

	// text formats
	static DecimalFormat timeFormat = new DecimalFormat ("00.0000");
	static DecimalFormat concFormat = new DecimalFormat ("0.0000");
	static DecimalFormat deflectionFormat = new DecimalFormat ("0.0000");
	static DecimalFormat expFormat = new DecimalFormat ("0.000E0");
	
	
	public SimFil_Graphics_( ) {
		
		if (!Env.remote) { 
			// make InfoCCD classes, if desired.. 
			// the boolean arguments specify optional use of Console, Instructions, ParameterLoader
			new Info(true,true,true);
			
			makeAllControls(this); 
		}
		
	}
	
	public static void makeAllControls(SimFil_Graphics_ pG) {
		mainControl = new MainControl_(pG);
		sphControl = new SphereControl_(pG);
		graphicsControl = new GraphicsControl_(pG);
		
		controlTab.addTab("main",mainControl.getContentPane());
		controlTab.addTab("stability tests",sphControl.getContentPane());
		controlTab.addTab("graphics",graphicsControl.getContentPane());
		
		controlPanel.add(controlTab);
		controlFrame.getContentPane().add(controlPanel);
		// set location and size of controlFrame
		controlFrame.setSize(controlFrame.getPreferredSize());
		//controlFrame.setLocationRelativeTo(null);
		controlFrame.setLocation(Env.frameWidth.getIntValue(),0);
		controlFrame.setVisible(true);

	}
	
	public static void updateAllControls() {
		graphicsControl.updateInfoFields();
		mainControl.updateInfoFields();
		sphControl.updateInfoFields();
	}


	/**
	 *  Initializes the Example by parsing command-line arguments,
	 *  building an AWT Frame, constructing a menubar, and creating
	 *  the 3D canvas.
	 *
	 *  @param   args    a String array of command-line arguments
	 */
	protected void initialize() {
		// Build the frame
		theFrame = new Frame( );
		theFrame.setBackground(Env.universeColor);
		theFrame.setSize( Env.frameWidth.getIntValue(), Env.frameHeight.getIntValue() );
		theFrame.setLayout( new BorderLayout( ) );
		theFrame.setTitle( "SimFil - in silico flexible filaments" );

		// Set up a close behavior
		theFrame.addWindowListener( this );
		
		// Create a canvas
		theCanvas = new CapturingCanvas3D_(SimpleUniverse.getPreferredConfiguration());
		theCanvas.setSize( Env.frameWidth.getIntValue(), Env.frameHeight.getIntValue() );
		theFrame.add( "Center", theCanvas );

		// Create time panel
		//makeTopPanel();
		//theFrame.add( "North", topPanel );
		
		// Create bottom panel
		makeBottomPanel();
		//theFrame.add("South",bottomPanel);
		
		// Pack
		theFrame.pack( );
		theFrame.validate( );
	
		makeMenuBar();
		
		// set frameHeadSize for future canvas/jframe resizing
		frameHeadSize = theFrame.getHeight() - theCanvas.getHeight();
		//talkln ("frameHeadSize: " + frameHeadSize);
	
	}



	//--------------------------------------------------------------
	//  SCENE CONTENT
	//--------------------------------------------------------------

	/**
	 *  Builds the 3D universe by constructing a virtual universe
	 *  (via SimpleUniverse), a view platform (via SimpleUniverse), and
	 *  a view (via SimpleUniverse).  A headlight is added and a
	 *  set of behaviors initialized to handle navigation types.
	 */
	protected void buildUniverse( ) {
		SimpleUniverse universe = new SimpleUniverse(theCanvas);
		
		ViewingPlatform viewingPlatform = universe.getViewingPlatform( );
		theViewTransform = viewingPlatform.getViewPlatformTransform( );


		//
		//  Create a "headlight" as a forward-facing directional light.
		//  Set the light's bounds to huge.  Since we want the light
		//  on the viewer's "head", we need the light within the
		//  TransformGroup containing the ViewPlatform.  The
		//  ViewingPlatform class creates a handy hook to do this
		//  called "platform geometry".  The PlatformGeometry class is
		//  subclassed off of BranchGroup, and is intended to contain
		//  a description of the 3D platform itself... PLUS a headlight!
		//  So, to add the headlight, create a new PlatformGeometry group,
		//  add the light to it, then add that platform geometry to the
		//  ViewingPlatform.
		//
		BoundingSphere allBounds = new BoundingSphere(
			new Point3d( 0.0, 0.0, 0.0 ), 100000.0 );

		PlatformGeometry pg = new PlatformGeometry( );
		headlight = new DirectionalLight( );
		headlight.setColor( Env.headlightColor );
		headlight.setDirection( new Vector3f( 0.0f, 0.0f, -1.0f ) );
		headlight.setInfluencingBounds( allBounds );
		headlight.setCapability( Light.ALLOW_STATE_WRITE );
		headlight.setCapability( Light.ALLOW_COLOR_WRITE );
		pg.addChild( headlight );
		
		bugEnd1Light = new PointLight();
		bugEnd1Light.setCapability(Light.ALLOW_STATE_WRITE);
		bugEnd1Light.setCapability(Light.ALLOW_COLOR_WRITE);
		bugEnd1Light.setColor(Env.bugEndLightColor3f);
		bugEnd1Light.setInfluencingBounds(allBounds);
		bugEnd1Light.setAttenuation(5.0f,0f,0.0f);
		bugEnd1Light.setPosition(-1f,0f,0f);
		bugEnd1Light.setEnable(bugEndLightsOn);
		pg.addChild(bugEnd1Light);
		
		bugEnd2Light = new PointLight();
		bugEnd2Light.setCapability(Light.ALLOW_STATE_WRITE);
		bugEnd2Light.setCapability(Light.ALLOW_COLOR_WRITE);
		bugEnd2Light.setColor(Env.bugEndLightColor3f);
		bugEnd2Light.setInfluencingBounds(allBounds);
		bugEnd2Light.setAttenuation(5.0f,0f,0.0f);
		bugEnd2Light.setPosition(1f,0f,0f);
		bugEnd2Light.setEnable(bugEndLightsOn);
		pg.addChild(bugEnd2Light);
		
		viewingPlatform.setPlatformGeometry( pg );

		//
		//  Create the 3D content BranchGroup, containing:
		//
		//    - a TransformGroup who's transform the examine behavior
		//      will change (when enabled).
		//
		//    - 3D geometry to view
		//
		// Build the scene root
		BranchGroup sceneRoot = new BranchGroup( );

		// Build a transform that we can modify
		theSceneTransform = new TransformGroup( );
		theSceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ );
		theSceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE );
		theSceneTransform.setCapability(Group.ALLOW_CHILDREN_EXTEND );


		//
		//  Build the scene, add it to the transform, and add
		//  the transform to the scene root
		//
		Group scene = this.buildScene( );
		theSceneTransform.addChild( scene );
		sceneRoot.addChild( theSceneTransform );


		examineBehavior = new ExamineViewerBehavior_(theSceneTransform, theFrame ); 
		examineBehavior.setSchedulingBounds( allBounds );
		sceneRoot.addChild( examineBehavior );

		examineBehavior.setEnable( true );


		//
		//  Compile the scene branch group and add it to the SimpleUniverse.
		//
		if ( shouldCompile ) { sceneRoot.compile( ); }
		universe.addBranchGraph( sceneRoot );
		resetView();
		
		// antialias
		theCanvas.getView().setSceneAntialiasingEnable(true);
	}
	
	public boolean canvasSizeCurrent () {
		if (theCanvas.getHeight() != Env.frameHeight.getIntValue()) { return false; }
		if (theCanvas.getWidth() != Env.frameWidth.getIntValue()) { return false; }
		return true;
	}
	
	public void makeCanvasSizeCurrent () {
		setCanvasSize (Env.frameWidth.getIntValue(), Env.frameHeight.getIntValue());
	}
	
	public void updateCanvasSizeParams() {
		Env.frameWidth.setValue(theFrame.getWidth());
		Env.frameHeight.setValue(theFrame.getHeight()-frameHeadSize);
	}

	public void setCanvasSize (int width, int height) {
		//talkln ("Resetting canvas size to " + width + "," + height);
		theCanvas.stopRenderer();
		theFrame.remove(theCanvas);
		theFrame.setSize(width,height+frameHeadSize);
		theCanvas.setSize(width,height);
		theFrame.add("Center",theCanvas);
		theCanvas.startRenderer();
		
		//topPanel.setSize(width,height);
		//topPanel.remove(timeLabel);
		//topPanel.add(timeLabel);
		//topPanel.add(concLabel);
	}
	
	//  Reset transforms
	public static void resetView( )
	{
		Transform3D trans = new Transform3D( );
		trans.setScale(Env.transScale.getValue());
		theSceneTransform.setTransform( trans );
		trans.set( new Vector3f( 0.0f, 0.0f, 30.0f ) );
		theViewTransform.setTransform( trans );
	}
	
	public void setRotZ90View( )
	{
		Transform3D trans = new Transform3D( );
		Transform3D rotTrans = new Transform3D();
		rotTrans.rotZ(Math.PI/2);
		trans.mul(rotTrans);
		trans.setScale(Env.transScale.getValue());
		theSceneTransform.setTransform( trans );
		trans.set( new Vector3f( 0.0f, 0.0f, 30.0f ) );
		theViewTransform.setTransform( trans );
	}
	
	public void rotateView() {
		double rotAng = 1*Math.PI/180;
		Transform3D rotT3D = new Transform3D();
		rotT3D.rotY(rotAng);
		Transform3D curT3D = new Transform3D();
		theSceneTransform.getTransform(curT3D);
		curT3D.mul(rotT3D);
		curT3D.setScale(Env.transScale.getValue());
		theSceneTransform.setTransform(curT3D);
	}
	
	public void rotateView(double rotAng) {
		rotAng = rotAng*Math.PI/180;
		Transform3D rotT3D = new Transform3D();
		rotT3D.rotY(rotAng);
		Transform3D curT3D = new Transform3D();
		theSceneTransform.getTransform(curT3D);
		curT3D.mul(rotT3D);
		curT3D.setScale(Env.transScale.getValue());
		theSceneTransform.setTransform(curT3D);
	}


	private BranchGroup initialBugScene() {
		bugState = new BranchGroup();
		allBranchGroupCapabilities(bugState);
		
		for (int i=0;i<Thing.thingCt;i++) {
			bugState.addChild(Thing.theThings[i].getGraphicsNode());
		}
		return bugState;
	}
	
	public static void updateBugScene() {
		Thing curThing;
		//if (bugState != null) { bugState.detach();}
		//bugState = new BranchGroup();
		//allBranchGroupCapabilities(bugState);

		for (int i=0;i<Thing.thingCt;i++) {
			curThing = Thing.theThings[i];
			if (!curThing.graphicsMade) {
				bugState.addChild(curThing.getGraphicsNode());
			} else {
				curThing.updateGraphics();
			}
		}
	
		//scene.addChild(bugState);
	}
	
	public static void updateQKBugScene() {
		Thing curThing;
		
		if (Env.filRenderOff) {
			for (int i=0;i<PSphere.sphereCt+1;i++) {
				curThing = Thing.theThings[i];
				if (!curThing.graphicsMade) {
					bugState.addChild(curThing.getGraphicsNode());
				} else {
					curThing.updateGraphics();
				}
			}
			return;
		} 
		

		for (int i=0;i<Thing.thingCt;i++) {
			curThing = Thing.theThings[i];
			if (!curThing.graphicsMade) {
				bugState.addChild(curThing.getGraphicsNode());
			} else {
				curThing.updateGraphics();
			}
		}
		
	}
	
	
	public static void clearForRun () {
		synchronized(Env.safeO) {
			FilSegment.removeAll();
			PSphere.removeAll();
			Thing.removeDeadThings();
			Thing.thingCt = 0;	// no Things left
			if (!Env.remote) {
				//updateBugScene();
				updateAllControls();
			}
			Env.simulationTime = 0;
		}
	}
	
	public static void clearForRender () {
		synchronized(Env.safeO) {
			FilSegment.removeAll();
			PSphere.removeAll();
			Thing.removeDeadThings();
			Thing.thingCt = 0;	// no Things left
		}
	}
	
	public static void captureImage (CapturingCanvas3D_ theCanvas) {
		theCanvas.writeJPEG_ = true;
		theCanvas.repaint();
	}
	
	public static void waitOnCapture (CapturingCanvas3D_ theCanvas) {
		while (theCanvas.isRendering()) {
			try { Thread.sleep(500); } catch (InterruptedException e) { talkln ("error sleeping"); }
		}
	}
	
	//
	//  Build scene
	//
	public Group buildScene( )
	{
		
		// Build the scene root
		scene = new Group( );

		// Create application bounds
		BoundingSphere worldBounds = new BoundingSphere(
			new Point3d( 0.0, 0.0, 0.0 ),  // Center
			1000.0 );                      // Extent

		// Set the background color and its application bounds
		background = new Background( );
		background.setColor(Env.universeColor3f);
		background.setCapability( Background.ALLOW_IMAGE_WRITE );
		background.setApplicationBounds( worldBounds );
		scene.addChild( background );

		// Build foreground geometry
		scene.addChild( initialBugScene( ) );

		return scene;
	}


	public void makeTopPanel () {
		topPanel = new JPanel();
		topPanel.setSize(Env.frameWidth.getIntValue(),8);
		topPanel.setLayout(new GridLayout(1,4));
		topPanel.setBackground(Env.controlBackColor);
		
		timeLabel = new JLabel(" ");
		timeLabel.setForeground(Env.controlForeColor);
		topPanel.add(timeLabel);
		
		concLabel = new JLabel(" ");
		concLabel.setForeground(Env.controlForeColor);
		topPanel.add(concLabel);
		
		filCtLabel = new JLabel(" ");
		filCtLabel.setForeground(Env.controlForeColor);
		topPanel.add(filCtLabel);
		
		monCtLabel = new JLabel(" ");
		monCtLabel.setForeground(Env.controlForeColor);
		topPanel.add(monCtLabel);
		
	}
	
	public void makeBottomPanel() {
		bottomPanel = new JPanel();
		bottomPanel.setSize(Env.frameWidth.getIntValue(),bottomPanelHeight);
		bottomPanel.setLayout(new GridLayout(2,2));
		//bottomPanel.setBackground(Env.controlBackColor);
		JPanel subP1 = new JPanel();
		JPanel subP2 = new JPanel();
		
		JLabel headLightILabel = new JLabel("HeadLight Intensity:");
		headLightI = new JSlider(JSlider.HORIZONTAL,0,100,(int)(100*Env.headlightColor.x));
		headLightI.addChangeListener(this);
		subP1.add(headLightILabel);
		subP1.add(headLightI);
		
		JLabel bugLightILabel = new JLabel("   BugLight Intensity:");
		bugLightI = new JSlider(JSlider.HORIZONTAL,0,100,(int)(100*Env.bugEndLightColor3f.x));
		bugLightI.addChangeListener(this);
		subP2.add(bugLightILabel);
		subP2.add(bugLightI);
		
		bottomPanel.add(subP1);
		bottomPanel.add(subP2);
	}
	
	public static void setTopLabels () {
		setTimeLabel(); 
		setFilCtLabel();
	}
	
	public static String getTimeString() {
		return new String (" Time: " + String.valueOf(timeFormat.format(Env.simulationTime)) + " s ");
	}
	
	public static void setTimeLabel () {
		timeLabel.setText(getTimeString());
	}
		
	public static String getFilSegCtString() {
		int filCt;
		if (Env.fromQKFile) {
			filCt = FilSegment.filamentRenderCt;
		} else {
			filCt = FilSegment.filSegmentCt;
		}
		return new String (" FilSegs#: " + String.valueOf(filCt)); 
	}
	
	public static String getFilCtString() {
		int filCt;
		if (Env.fromQKFile) {
			filCt = FilSegment.filamentRenderCt;
		} else {
			filCt = FilSegment.filCt;
		}
		filCtString = " Filament#: " + String.valueOf(filCt);
		return filCtString; 
	}
	
	public static void setFilCtLabel () {
		filCtLabel.setText(getFilCtString());
	}
	
	public static String getTstFilLengthString() {
		return new String (" Length: " + String.valueOf(deflectionFormat.format(Env.filamentLength.getValue())) + " M "); 
	}
	
	public static String getCurrentDeflectionString() {
		return new String ("Current : " + String.valueOf(deflectionFormat.format(Env.currentDeflection)) + " M "); 
	}
	
	public static String getExpectedDeflectionString() {
		return new String ("Expected : " + String.valueOf(deflectionFormat.format(Env.expectedDeflection)) + " M "); 
	}
	
	public static String getRelaxationDispString() {
		if (!Env.relaxCantilever & !Env.relaxSimpleSupport) { Env.relaxationDeflection = Env.currentDeflection/Math.E; }
		return new String ("/e: " + String.valueOf(deflectionFormat.format(Env.relaxationDeflection)) + " M "); 
	}
	
	public static String getCurrentElasticity() {
		return new String ("kTorsion: " + String.valueOf(expFormat.format(Env.filTorqSpring.getValue())) + " N/rad ");
	}
	
	public static String getPersistenceTstLengthString () {
		return new String ("Length: " + String.valueOf(deflectionFormat.format(Env.persistenceTstLength)) + " M "); 
	}
	
	public static String getLastSegCorrelationString () {
		double lastSegCorr = 0;
		if (Env.counter > 0) { 
			double scaleVal = 1/Env.persistenceTrack[0].runningAverageVal();		// in case we haven't filled valuetracker yet
			lastSegCorr = scaleVal*Env.persistenceTrack[Env.persistenceSegNumber-1].runningAverageVal(); 
		}
		return new String (" LastSegCorrelation: " + String.valueOf(deflectionFormat.format(lastSegCorr))); 
	}
	
	public void makeMenuBar () {
		theMenuBar = new MenuBar();
		theFrame.setMenuBar( theMenuBar );

		// Add menu for program control
		Menu m = new Menu ("Program");
		m.addActionListener(this);
		
		runMenuItem = new CheckboxMenuItem ("Run",!Env.paused);
		runMenuItem.addItemListener(this);
		m.add(runMenuItem);
		
		pausedMenuItem = new CheckboxMenuItem ("Pause",Env.paused);
		pausedMenuItem.addItemListener(this);
		m.add(pausedMenuItem);
		
		m.add("Restart");
		
		m.addSeparator();
		
		writeFileMenuItem = new CheckboxMenuItem("Write JPEGs");
		writeFileMenuItem.addItemListener(this);
		writeFileMenuItem.setState(Env.toFile);
		m.add(writeFileMenuItem);
		
		logLpMenuItem = new CheckboxMenuItem("Log Segment Angular Correlations");
		logLpMenuItem.addItemListener(this);
		logLpMenuItem.setState(false);
		m.add(logLpMenuItem);

		m.addSeparator();
		
		m.add("Save Parameter Settings");
		m.add("Load Parameter Settings");
		
		theMenuBar.add(m);

		// View menu
		m = new Menu( "View" );
		m.addActionListener( this );
		
		m.add("Control Panel");
		
		m.addSeparator();
		
		paintOnMenuItem = new CheckboxMenuItem("Paint On");
		paintOnMenuItem.addItemListener(this);
		paintOnMenuItem.setState(Env.paintOn);
		m.add(paintOnMenuItem);
		
		rotateMenuItem = new CheckboxMenuItem("Rotate");
		rotateMenuItem.addItemListener(this);
		rotateMenuItem.setState(Env.viewRotation);
		m.add(rotateMenuItem);
		
		m.addSeparator();

		m.add( "Reset view" );
		
		theMenuBar.add( m );
		
		m = new Menu ( "TuningTasks" );
		
		m.add("--- Step 1: Choose tuning task and run ---");
		
		simpleSupportTstMenuItem = new CheckboxMenuItem("Simple support deflection / 1st Mode free time-constant");
		simpleSupportTstMenuItem.addItemListener(this);  
		simpleSupportTstMenuItem.setState(Env.simpleSupportElasticityTst);
		m.add(simpleSupportTstMenuItem);
		
		simpleSupportTstMenuItemMode2 = new CheckboxMenuItem("Simple support deflection / 2nd Mode free time-constant");
		simpleSupportTstMenuItemMode2.addItemListener(this);  
		simpleSupportTstMenuItemMode2.setState(Env.secondMode);
		m.add(simpleSupportTstMenuItemMode2);
		
		cantileverTstMenuItem = new CheckboxMenuItem("Cantilevered support deflection / 1st Mode free time-constant");
		cantileverTstMenuItem.addItemListener(this);  
		cantileverTstMenuItem.setState(Env.cantileverElasticityTst);
		m.add(cantileverTstMenuItem);
		
		m.addSeparator();
		
		persistenceTstMenuItem = new CheckboxMenuItem("Persistence length tuning");
		persistenceTstMenuItem.addItemListener(this);  
		persistenceTstMenuItem.setState(Env.persistenceTst);
		m.add(persistenceTstMenuItem);
		
		m.addSeparator();
		
		m.add("--- Step 2: When deflection is tuned, release constraints to measure time-constant ---");
		
		relaxConstraintsMenuItem = new CheckboxMenuItem("Relax Constraints");
		relaxConstraintsMenuItem.addItemListener(this);
		relaxConstraintsMenuItem.setState(Env.relaxSimpleSupport);
		m.add(relaxConstraintsMenuItem);
		
		theMenuBar.add(m);
		
		m = new Menu ( "PAIRS Stability Demos" );
		
		sphereInPlanesMenuItem = new CheckboxMenuItem("Test1: Small sphere jammed between two larger ones");
		sphereInPlanesMenuItem.addItemListener(this);
		sphereInPlanesMenuItem.setState(Env.sphereBetweenPlanes);
		m.add(sphereInPlanesMenuItem);
		
		spheresSqueezedMenuItem = new CheckboxMenuItem("Test2: Spheres squeezed in a container");
		spheresSqueezedMenuItem.addItemListener(this);
		spheresSqueezedMenuItem.setState(Env.spheresSqueezeFit);
		m.add(spheresSqueezedMenuItem);
		
		spheresInForceFieldMenuItem = new CheckboxMenuItem("Test3: Spheres under a constant force field");
		spheresInForceFieldMenuItem.addItemListener(this);
		spheresInForceFieldMenuItem.setState(Env.spheresUnderForceField);
		m.add(spheresInForceFieldMenuItem);
		
		spheresLinkedMenuItem = new CheckboxMenuItem("Test4: Spheres linked by a spring");
		spheresLinkedMenuItem.addItemListener(this);
		spheresLinkedMenuItem.setState(Env.spheresLinked);
		m.add(spheresLinkedMenuItem);
		
		filUnderTractionMenuItem = new CheckboxMenuItem("Test5: Filament under traction");
		filUnderTractionMenuItem.addItemListener(this);
		filUnderTractionMenuItem.setState(Env.filUnderTraction);
		m.add(filUnderTractionMenuItem);
		
		theMenuBar.add(m);
		
		m = new Menu ( "Special" );
		
		filMotionOffMenuItem = new CheckboxMenuItem("Filament Brownian Motion Off");
		filMotionOffMenuItem.addItemListener(this);
		filMotionOffMenuItem.setState(Env.filBrownianMotionOff);
		m.add(filMotionOffMenuItem);
		
		filCoordSysOnMenuItem = new CheckboxMenuItem("Render Coord. Sys. on Segments");
		filCoordSysOnMenuItem.addItemListener(this);
		filCoordSysOnMenuItem.setState(Env.filCoordSysOn);
		m.add(filCoordSysOnMenuItem);
		
		theMenuBar.add(m);
		
		theMenuBar.add(Info.infoMenu);
		
	}
	
	public void actionPerformed( ActionEvent event ) {
		String arg = event.getActionCommand( );
		
		if ( arg.equals( "Reset view" ) ) {
			resetView();
		}
		
		if ( arg.equals( "Reset view - RotZ90") ) {
			setRotZ90View();
		}
	
		if (arg.equals("Restart")) { 
			SimFil.setPaused();
			SimFil.restartRun(false); 
		}
		
		if ( arg.equals("Save Parameter Settings") ) {
			saveParamsToFile();
		}
		
		if ( arg.equals("Load Parameter Settings") ) {
			loadParamsFromFile();
		}
		
		if ( arg.equals("Control Panel") ) {
			controlFrame.setVisible(true);
		}
	
	}


	//
	//  Handle checkboxes
	//
	public void itemStateChanged( ItemEvent event )
	{
		Object src = event.getSource( );

		if ( src == paintOnMenuItem) {
			Env.paintOn = paintOnMenuItem.getState();
		}
		
		if (src == writeFileMenuItem) {
			if (writeFileMenuItem.getState()) { 
				prepForJPEGWriting(); 
			} else {
				Env.toFile = false;
				Env.toFilePath = null;
				Env.toFileName = null;
			}
		}
		
		if (src == logLpMenuItem) {
			if (logLpMenuItem.getState()) { 
				prepForLpWriting(); 
			} else {
				Env.toFile = false;
				Env.toFilePath = null;
				Env.toFileName = null;
			}
		}
		
		if (src == writeQKFileMenuItem) {
			if (writeQKFileMenuItem.getState()) { 
				prepForQKWriting(); 
			} else {
				Env.toQKFile = false;
				Env.qkFilePath = null;
				Env.outFileName = null;
			}
		}
		
		if (src == pausedMenuItem) {
			SimFil.setPaused();	
		}
		
		if (src == runMenuItem) {
			SimFil.setRunning();
		}
		
		if (src == rotateMenuItem) {
			Env.viewRotation = rotateMenuItem.getState();
		}
		
		
		if (src == filMotionOffMenuItem) {
			Env.filBrownianMotionOff = filMotionOffMenuItem.getState();
		}
		
		if (src == plasmidMotionOffMenuItem) {
			Env.plasmidBrownianMotionOff = plasmidMotionOffMenuItem.getState();
		}
		
		if (src == filCoordSysOnMenuItem) {
			Env.filCoordSysOn = filCoordSysOnMenuItem.getState();
		}
		
		if (src == relaxConstraintsMenuItem) {
			if (Env.simpleSupportElasticityTst) {
				Env.relaxSimpleSupport = relaxConstraintsMenuItem.getState();
				if (Env.relaxSimpleSupport) { FilSegment.simpleSupportRelaxActions(); }
			}
			if (Env.cantileverElasticityTst) {
				Env.relaxCantilever = relaxConstraintsMenuItem.getState();
				if (Env.relaxCantilever) { FilSegment.cantileveredRelaxActions(); }
			}
		}
		
		if (src == simpleSupportTstMenuItem) {
			if (simpleSupportTstMenuItem.getState()) {
				allTestsFalse();
				Env.simpleSupportElasticityTst = true;
				Env.secondMode = false;
				SimFil.restartRun(false);
			} else {
				simpleSupportTstMenuItem.setState(true);
			}
		}
		
		if (src == simpleSupportTstMenuItemMode2) {
			if (simpleSupportTstMenuItemMode2.getState()) {
				allTestsFalse();
				Env.simpleSupportElasticityTst = true;
				Env.secondMode = true;
				SimFil.restartRun(false);
			} else {
				simpleSupportTstMenuItemMode2.setState(true);
			}
		}
		
		if (src == cantileverTstMenuItem) {
			if (cantileverTstMenuItem.getState()) {
				allTestsFalse();
				Env.cantileverElasticityTst = true;
				SimFil.restartRun(false);
			} else {
				cantileverTstMenuItem.setState(true);
			}
		}
		
		if (src == persistenceTstMenuItem) {
			if (persistenceTstMenuItem.getState()) {
				allTestsFalse();
				Env.persistenceTst = true;
				SimFil.restartRun(false);
			} else {
				persistenceTstMenuItem.setState(true);
			}
		}
		
		if (src == sphereInPlanesMenuItem) {
			if (sphereInPlanesMenuItem.getState()) {
				allTestsFalse();
				Env.sphereBetweenPlanes = true;
				SimFil.restartRun(false);
			} else {
				sphereInPlanesMenuItem.setState(true);
			}
		}
		
		if (src == spheresInForceFieldMenuItem) {
			if (spheresInForceFieldMenuItem.getState()) {
				allTestsFalse();
				Env.spheresUnderForceField = true;
				SimFil.restartRun(false);
			} else {
				spheresInForceFieldMenuItem.setState(true);
			}
		}
		
		if (src == spheresSqueezedMenuItem) {
			if (spheresSqueezedMenuItem.getState()) {
				allTestsFalse();
				Env.spheresSqueezeFit = true;
				SimFil.restartRun(false);
			} else {
				spheresSqueezedMenuItem.setState(true);
			}
		}
		
		if (src == spheresLinkedMenuItem) {
			if (spheresLinkedMenuItem.getState()) {
				allTestsFalse();
				Env.spheresLinked = true;
				SimFil.restartRun(false);
			} else {
				spheresLinkedMenuItem.setState(true);
			}
		}
		
		if (src == filUnderTractionMenuItem) {
			if (filUnderTractionMenuItem.getState()) {
				allTestsFalse();
				Env.filUnderTraction = true;
				SimFil.restartRun(false);
			} else {
				filUnderTractionMenuItem.setState(true);
			}
		}
	}
	
	public void allTestsFalse () {
		Env.simpleSupportElasticityTst = false;
		Env.cantileverElasticityTst = false;
		Env.persistenceTst = false;
		Env.sphereBetweenPlanes = false;
		Env.spheresUnderForceField = false;
		Env.spheresSqueezeFit = false;
		Env.spheresLinked = false;
		Env.filUnderTraction = false;
	}
	
	public static void resetTuningTestsState () {
		filMotionOffMenuItem.setState(Env.filBrownianMotionOff);
		relaxConstraintsMenuItem.setState(Env.relaxSimpleSupport);
		persistenceTstMenuItem.setState(Env.persistenceTst);
		if (Env.simpleSupportElasticityTst) {
			simpleSupportTstMenuItem.setState(!Env.secondMode);
			simpleSupportTstMenuItemMode2.setState(Env.secondMode);
		} else {
			simpleSupportTstMenuItem.setState(false);
			simpleSupportTstMenuItemMode2.setState(false);
		}
		cantileverTstMenuItem.setState(Env.cantileverElasticityTst);
		sphereInPlanesMenuItem.setState(Env.sphereBetweenPlanes);
		spheresInForceFieldMenuItem.setState(Env.spheresUnderForceField);
		spheresSqueezedMenuItem.setState(Env.spheresSqueezeFit);
		spheresLinkedMenuItem.setState(Env.spheresLinked);
		filUnderTractionMenuItem.setState(Env.filUnderTraction);
	}
	
	public void stateChanged(ChangeEvent e) {
	    JSlider source = (JSlider)e.getSource();
	    
	    if (source == headLightI) {
	    	if (!source.getValueIsAdjusting()) {
		       float newHeadLightI = (float)(source.getValue())/100;
		       headlight.setColor(new Color3f(newHeadLightI,newHeadLightI,newHeadLightI));
		    }
	    }
	    
	    if (source == bugLightI) {
	    	if (!source.getValueIsAdjusting()) {
		       float newBugLightI = (float)(source.getValue())/100;
		       bugEnd1Light.setColor(new Color3f(newBugLightI,newBugLightI,newBugLightI));
		       bugEnd2Light.setColor(new Color3f(newBugLightI,newBugLightI,newBugLightI));
		    }
	    }
	    
	    if (source == bugLightI) {
	    	if (!source.getValueIsAdjusting()) {
		       float newBugLightI = (float)(source.getValue())/100;
		       bugEnd1Light.setColor(new Color3f(newBugLightI,newBugLightI,newBugLightI));
		       bugEnd2Light.setColor(new Color3f(newBugLightI,newBugLightI,newBugLightI));
		    }
	    }
	}
	
	
	public static void allBranchGroupCapabilities(BranchGroup bg) {
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		bg.setCapability(BranchGroup.ALLOW_DETACH);
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
	
	public void loadParamsFromFile () {
		String loadFromName = null;
		loadFromName = FileOps_.getFileToLoad("Load parameter settings from ...", this);
		if (loadFromName == null) {
			talkln ("Parameter settings not loaded.");
		} else {
			File loadFromFile = new File(loadFromName);
			FileOps_.loadParamConfig(loadFromFile, true);
		}
	}
	
	public void saveParamsToFile () {
		String saveToName = null;
		saveToName = FileOps_.getFileToSave("Save parameter settings to ...", this);
		if (saveToName == null) {
			talkln ("Parameter settings not saved.");
		} else {
			File saveToFile = new File(saveToName);
			FileOps_.writeParamConfig(saveToFile);
		}
	}
	
	public void prepForJPEGWriting () {
		Env.toFilePath = null;
		Env.toFilePath = FileOps_.getDirectoryToSave("Create folder for JPEG files ...", this);
		if (Env.toFilePath == null) { 
			Env.toFile = false;
			writeFileMenuItem.setState(false);
		} else {
			int lastbitIndex = Env.toFilePath.lastIndexOf(File.separator);
			Env.toFileName = Env.toFilePath.substring(lastbitIndex+1);
			theCanvas.capturePath = Env.toFilePath;
			theCanvas.captureName = Env.toFileName;
			Env.toFile = true;
		}
	}
	
	public void prepForLpWriting () {
		String lpFilePath = null;
		lpFilePath = FileOps_.getFileToSave("Create angular correlations file ... ", this);
		if (lpFilePath == null) { 
			Env.justLogLp = false;
			logLpMenuItem.setState(false);
		} else {
			FileOps_.mkPersistenceLengthFile(lpFilePath);
			Env.justLogLp = true;
		}
	}
	
	public static void prepForQKWriting () {
		Env.qkFilePath = null;
		Env.qkFilePath = FileOps_.getDirectoryToSave("Create folder for QK files ...", null);
		if (Env.qkFilePath == null) { 
			Env.toQKFile = false;
			writeQKFileMenuItem.setState(false);
		} else {
			int lastbitIndex = Env.qkFilePath.lastIndexOf(File.separator);
			Env.outFileName = Env.qkFilePath.substring(lastbitIndex+1);
			Env.toQKFile = true;
		}
	}
	
	public static void prepForQKRender () {
		Env.fromQKFilePath = null;
		Env.fromQKFilePath = FileOps_.getDirectoryToLoad("Select folder of QK files to render ...", null);
		if (Env.fromQKFilePath == null) { 
			Env.fromQKFileName = null;
			Env.fromQKFile = false;
		} else {
			int lastbitIndex = Env.fromQKFilePath.lastIndexOf(File.separator);
			Env.fromQKFileName = Env.fromQKFilePath.substring(lastbitIndex+1);
			Env.fromQKFile = true;
		}
	}
	
	public static void prepForQKStateLoad () {
		Env.fromQKStatePath = null;
		Env.fromQKStatePath = FileOps_.getFileToLoad("Select QK file state to load ...", null);
	}


	//--------------------------------------------------------------
	//  USER INTERFACE
	//--------------------------------------------------------------


	/**
	 *  Shows the application's frame, making it and its menubar,
	 *  3D canvas, and 3D content visible.
	 */
	public void showFrame( )
	{
		theFrame.setVisible(true);
	}


	/**
	 *  Quits the application.
	 */
	public void quit( )
	{
		System.exit( 0 );
	}



	/**
	 *  Handles checkbox items on a CheckboxMenu.  The Example class has
	 *  none of its own, but subclasses may have some.
	 *
	 *  @param   menu    which CheckboxMenu needs action
	 *  @param   check   which CheckboxMenu item has changed
	 */
	public void checkboxChanged( CheckboxMenu_ menu, int check )
	{
		// None for us
	}


	/**
	 *  Handles a window closing event notifying the application
	 *  that the user has chosen to close the application without
	 *  selecting the "Exit" menu item.
	 *
	 *  @param   event   a WindowEvent indicating the window is closing
	 */
	public void windowClosing( WindowEvent event )
	{
		quit( );
	}
	public void windowClosed( WindowEvent event )
	{
	}
	public void windowOpened( WindowEvent event )
	{
	}
	public void windowIconified( WindowEvent event )
	{
	}
	public void windowDeiconified( WindowEvent event )
	{
	}
	public void windowActivated( WindowEvent event )
	{
	}
	public void windowDeactivated( WindowEvent event )
	{
	}



}